@@ -1,4 +1,4 @@
-/* $EPIC: commands.c,v 1.210 2013/07/31 04:17:53 jnelson Exp $ */
+/* $EPIC: commands.c,v 1.212 2013/11/01 04:37:41 jnelson Exp $ */
 /*
  * commands.c -- Stuff needed to execute commands in ircII.
  *		 Includes the bulk of the built in commands for ircII.
@@ -6,7 +6,7 @@
  * Copyright (c) 1990 Michael Sandroff.
  * Copyright (c) 1991, 1992 Troy Rollo.
  * Copyright (c) 1992-1996 Matthew Green.
- * Copyright  1995, 2010 EPIC Software Labs
+ * Copyright 1995, 2010 EPIC Software Labs
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -105,6 +105,7 @@ static	void	breakcmd	(const char *, char *, const char *);
 static	void	commentcmd 	(const char *, char *, const char *);
 static	void	continuecmd	(const char *, char *, const char *);
 static	void	ctcp 		(const char *, char *, const char *);
+	void	debuglogcmd	(const char *, char *, const char *);
 static	void	deop 		(const char *, char *, const char *);
 static	void	send_to_channel_first	(const char *, char *, const char *);
 static	void	send_to_query_first	(const char *, char *, const char *);
@@ -212,6 +213,7 @@ static	IrcCommand irc_command[] =
 	{ "CONTINUE",	continuecmd	},
 	{ "CTCP",	ctcp		},
 	{ "DCC",	dcc_cmd		}, /* dcc.c */
+	{ "DEBUGLOG",	debuglogcmd	}, /* debuglog.c */
 	{ "DEFER",	defercmd	},
 	{ "DEOP",	deop		},
 	{ "DESCRIBE",	describe	},
@@ -442,6 +444,11 @@ BUILT_IN_COMMAND(cd)
 		say("The current directory is invalid.");
 }
 
+BUILT_IN_COMMAND(e_call)
+{
+	dump_call_stack();
+}
+
 /* clear: the CLEAR command.  Figure it out */
 BUILT_IN_COMMAND(e_clear)
 {
@@ -528,6 +535,8 @@ BUILT_IN_COMMAND(ctcp)
 		say("Usage: /CTCP <[=]nick|channel|*> [<request>]");
 }
 
+#include "debuglog.c"
+
 struct defer {
 	char *	cmds;
 	int	servref;
@@ -1432,6 +1441,26 @@ BUILT_IN_COMMAND(info)
 	send_to_server("INFO %s", args?args:empty_string);
 }
 
+/*
+ * eval_inputlist:  Cute little wrapper that calls runcmds() when we
+ * get an input prompt ..
+ */
+static void	eval_inputlist (char *args, char *line)
+{
+        char *tmp;
+	if (args[0]=='(') {
+                if (!(tmp = next_expr(&args, '('))) {
+			yell("INPUT: syntax error with arglist");
+			return;
+		}
+                runcmds_with_arglist(args, tmp, line);
+	} else {
+                /* traditional behavior */
+		runcmds(args, line);
+	}
+}
+
+
 /*
  * inputcmd:  the INPUT command.   Takes a couple of arguements...
  * the first surrounded in double quotes, and the rest makes up
@@ -1451,6 +1480,8 @@ BUILT_IN_COMMAND(inputcmd)
 		argument = next_arg(args, &args);
 		if (!my_stricmp(argument, "-noecho"))
 			echo = 0;
+		if (!my_stricmp(argument, "--"))
+			break;
 	}
 
 	if (!(prompt = new_next_arg(args, &args)))
@@ -1487,8 +1518,8 @@ BUILT_IN_COMMAND(license)
 	yell("Copyright (c) 1990 Michael Sandroff.");
 	yell("Copyright (c) 1991, 1992 Troy Rollo.");
  	yell("Copyright (c) 1992-1996 Matthew Green.");
-	yell("Copyright  1994 Jake Khuon.");
-	yell("Coypright  1993, 2013 EPIC Software Labs.");
+	yell("Copyright 1994 Jake Khuon.");
+	yell("Coypright 1993, 2013 EPIC Software Labs.");
 	yell("All rights reserved");
 	yell(" ");
 	yell("Redistribution and use in source and binary forms, with or");
@@ -1631,7 +1662,6 @@ BUILT_IN_COMMAND(load)
 	char *	sargs;
 	char *	use_path;
 	char *	expanded;
-	FILE *	fp;
         struct epic_loadfile * elf;
 	int	display;
 	int	do_one_more = 0;
@@ -1640,12 +1670,6 @@ BUILT_IN_COMMAND(load)
 	off_t	file_contents_size = 0;
 	char *	declared_encoding = NULL;
 
-        int             idx;
-        int             err;
-        char            errstr[1024];
-        char            buf[8192];
-
-
 	if (++load_depth == MAX_LOAD_DEPTH)
 	{
 		load_depth--;
@@ -2946,7 +2970,7 @@ BUILT_IN_COMMAND(typecmd)
                                 break;
                 }
 
-                edit_char(key);
+		edit_char(key);
         }
 }
 
@@ -3415,30 +3439,6 @@ void 	new_send_text (int server, const char *orig_target_list, const char *text,
 }
 #endif
 
-/*
- * eval_inputlist:  Cute little wrapper that calls runcmds() when we
- * get an input prompt ..
- */
-static void	eval_inputlist (char *args, char *line)
-{
-        char *tmp;
-	if (args[0]=='(') {
-                if (!(tmp = next_expr(&args, '('))) {
-			yell("INPUT: syntax error with arglist");
-			return;
-		}
-                runcmds_with_arglist(args, tmp, line);
-	} else {
-                /* traditional behavior */
-		runcmds(args, line);
-	}
-}
-
-BUILT_IN_COMMAND(e_call)
-{
-	dump_call_stack();
-}
-
 /***************************************************************************/
 static char *	parse_line_alias_special (const char *name, const char *what, char *args, void *arglist, int function);
 
