@@ -1,4 +1,4 @@
-/* $EPIC: parse.c,v 1.95 2012/11/24 01:42:51 jnelson Exp $ */
+/* $EPIC: parse.c,v 1.98 2014/01/12 01:05:11 jnelson Exp $ */
 /*
  * parse.c: handles messages from the server.   Believe it or not.  I
  * certainly wouldn't if I were you. 
@@ -1411,8 +1411,8 @@ protocol_command rfc1459[] = {
 { 	"CONNECT",	NULL,		0		},
 {	"ERROR",	p_error,	0		},
 {	"ERROR:",	p_error,	0		},
-{	"INVITE",	p_invite,	0		},
 {	"INFO",		NULL,		0		},
+{	"INVITE",	p_invite,	0		},
 {	"ISON",		NULL,		PROTO_QUOTEBAD	},
 {	"JOIN",		p_channel,	0		},
 {	"KICK",		p_kick,		0		},
@@ -1439,8 +1439,8 @@ protocol_command rfc1459[] = {
 {	"STATS",	NULL,		0		},
 {	"SUMMON",	NULL,		0		},
 {	"TIME",		NULL,		0		},
-{	"TRACE",	NULL,		0		},
 {	"TOPIC",	p_topic,	0		},
+{	"TRACE",	NULL,		0		},
 {	"USER",		NULL,		0		},
 {	"USERHOST",	NULL,		PROTO_QUOTEBAD	},
 {	"USERS",	NULL,		0		},
@@ -1454,6 +1454,10 @@ protocol_command rfc1459[] = {
 #define NUMBER_OF_COMMANDS (sizeof(rfc1459) / sizeof(protocol_command)) - 2;
 int 	num_protocol_cmds = -1;
 
+#define islegal(c) ((((c) >= 'A') && ((c) <= '}')) || \
+                    (((c) >= '0') && ((c) <= '9')) || \
+                     ((c) & 0x80))
+
 /*
  * parse_server: parses messages from the server, doing what should be done
  * with them 
@@ -1503,6 +1507,12 @@ void 	parse_server (const char *orig_line, size_t orig_line_size)
 		return;		/* Serious protocol violation -- ByeBye */
 	}
 
+	if (*from && !islegal(*from))
+	{ 
+		rfc1459_odd(from, comm, ArgList);
+		return;		
+	}
+
 	/* Some day all this needs to be replaced with an alist. */
 	if (is_number(comm))
 		numbered_command(from, comm, ArgList);
