@@ -1,4 +1,4 @@
-/* $EPIC: server.c,v 1.257 2013/07/28 23:16:14 jnelson Exp $ */
+/* $EPIC: server.c,v 1.259 2013/11/06 03:49:33 jnelson Exp $ */
 /*
  * server.c:  Things dealing with that wacky program we call ircd.
  *
@@ -314,6 +314,10 @@ static	int	str_to_serverinfo (char *str, ServerInfo *s)
 			break;
 	    }
 
+		/* I don't see how this is possible, but clang says it is */
+		if (descstr == NULL)
+			break;
+
 		/* And advance to the next field */
 		if ((span = findchar_quoted(descstr, ':')) >= 0)
 		{
@@ -3418,6 +3422,8 @@ void	set_server_005 (int refnum, char *setting, const char *value)
 	    else
 		set_server_stricmp_table(refnum, 1);
 	}
+
+	update_all_status();
 }
 
 static char *	get_all_server_groups (void)
